import datetime
from passlib.totp import TOTP

UTC = datetime.timezone.utc
EPOCH_START = datetime.datetime(1970, 1, 1, tzinfo=UTC)

def make_epoch(*args):
    dt = datetime.datetime(*args, tzinfo=UTC)
    # XXX: think "%s" is broken, adjusts dt for host's timezone
    # return int(dt.strftime("%s"))
    return (dt - EPOCH_START).total_seconds()

def parse_epoch(value):
    # NOTE: either method should return same result...
    # return datetime.datetime.utcfromtimestamp(value).replace(tzinfo=UTC)
    return datetime.datetime.fromtimestamp(value, UTC)

token_base = TOTP(
    key="a" * 16,
    alg='sha256',
    period=86400,  # 1 day
    digits=6,
)

ref_time = make_epoch(2017, 12, 5)
print("ref time=%r; parse_epoch=%r; expected (2017, 12, 5, 0, 0, 0)" %
      (ref_time, parse_epoch(ref_time)))

token_raw = token_base.generate(time=ref_time)

print("start=%r; parse_epoch=%r; expected (2017, 12, 5, 0, 0, 0)" %
      (token_raw.start_time, parse_epoch(token_raw.start_time)))

print("expire=%r; parse_epoch=%r; expected (2017, 12, 6, 0, 0, 0)" %
      (token_raw.expire_time, parse_epoch(token_raw.expire_time)))
